from typing import Any, Dict, List, Optional

from clipped.utils.lists import to_list
from clipped.utils.sanitizers import sanitize_value

from polyaxon.containers.names import sanitize_container_name
from polyaxon.docker import docker_types
from polyaxon.runner.converter import BaseConverter
from polyaxon.runner.converter.common.containers import sanitize_container_command_args


class ContainerMixin(BaseConverter):
    @classmethod
    def _patch_container(
        cls,
        container: docker_types.V1Container,
        name: Optional[str] = None,
        command: Optional[List[str]] = None,
        args: Optional[List[str]] = None,
        image: Optional[str] = None,
        image_pull_policy: Optional[str] = None,
        env: Optional[List[docker_types.V1EnvVar]] = None,
        env_from: Optional[List[Any]] = None,
        volume_mounts: Optional[List[docker_types.V1VolumeMount]] = None,
        ports: Optional[List[docker_types.V1ContainerPort]] = None,
        resources: Optional[docker_types.V1ResourceRequirements] = None,
    ) -> docker_types.V1Container:
        container.name = sanitize_container_name(name or container.name)
        container.env = to_list(container.env, check_none=True) + to_list(
            env, check_none=True
        )
        container.volume_mounts = to_list(
            container.volume_mounts, check_none=True
        ) + to_list(volume_mounts, check_none=True)
        container.ports = to_list(container.ports, check_none=True) + to_list(
            ports, check_none=True
        )
        container.resources = container.resources or resources
        container.image = container.image or image

        if not any([container.command, container.args]):
            container.command = command
            container.args = args

        return cls._sanitize_container(container)

    @staticmethod
    def _sanitize_container_env(
        env: List[docker_types.V1EnvVar],
    ) -> List[docker_types.V1EnvVar]:
        def sanitize_env_dict(d: Dict):
            return docker_types.V1EnvVar(
                __root__={
                    d_k: sanitize_value(d_v, handle_dict=False)
                    for d_k, d_v in d.items()
                }
            )

        results = []
        for e in env or []:
            if isinstance(e, dict):
                e = sanitize_env_dict(e)
                results.append(e)
            elif isinstance(e, tuple):
                if e[1] is not None:
                    e = docker_types.V1EnvVar(
                        __root__=(e[0], sanitize_value(e[1], handle_dict=False))
                    )
                results.append(e)
            elif isinstance(e, docker_types.V1EnvVar):
                results.append(e)

        return results

    @classmethod
    def _sanitize_container(
        cls,
        container: docker_types.V1Container,
    ) -> docker_types.V1Container:
        container = sanitize_container_command_args(container)
        container.env = cls._sanitize_container_env(container.env)
        return container



import sys

from typing import Type

from clipped.formatting import Printer

from polyaxon.config.manager import ConfigManager
from polyaxon.schemas.responses.v1_project import V1Project
from polyaxon.utils import cli_constants


class ProjectConfigManager(ConfigManager):
    """Manages project configuration .project file."""

    VISIBILITY = ConfigManager.Visibility.ALL
    IN_PROJECT_DIR = True
    CONFIG_FILE_NAME = ".project"
    CONFIG: Type[V1Project] = V1Project

    @classmethod
    def get_config_or_raise(cls) -> V1Project:
        project = cls.get_config()
        if not project:
            Printer.error(
                "No project was found, please initialize a project."
                " {}".format(cli_constants.INIT_COMMAND)
            )
            sys.exit(1)

        return project



import os

from typing import List, Optional

from clipped.utils.json import orjson_dumps
from clipped.utils.lists import to_list

from polyaxon.exceptions import PolyaxonConverterError

REPO_INIT_COMMAND = ["polyaxon", "initializer", "git"]


def get_repo_context_args(
    name: str,
    url: str,
    revision: str,
    mount_path: str,
    connection: Optional[str] = None,
    flags: Optional[List[str]] = None,
) -> List[str]:
    if not name:
        raise PolyaxonConverterError(
            "A repo name is required to create a repo context."
        )
    if not url:
        raise PolyaxonConverterError("A repo url is required to create a repo context.")

    args = [
        "--repo-path={}".format(os.path.join(mount_path, name)),
        "--url={}".format(url),
    ]

    if revision:
        args.append("--revision={}".format(revision))

    if connection:
        args.append("--connection={}".format(connection))

    flags = to_list(flags, check_none=True)
    if flags:
        args.append("--flags={}".format(orjson_dumps(flags)))
    return args



import datetime

from typing import Any, Dict, Optional

from clipped.types.email import EmailStr
from pydantic import StrictStr

from polyaxon.schemas.base import BaseResponseModel


class V1Organization(BaseResponseModel):
    user: Optional[StrictStr]
    user_email: Optional[EmailStr]
    name: Optional[StrictStr]
    is_public: Optional[bool]
    created_at: Optional[datetime.datetime]
    updated_at: Optional[datetime.datetime]
    support_revoke_at: Optional[datetime.datetime]
    expiration: Optional[int]
    role: Optional[StrictStr]
    queue: Optional[StrictStr]
    preset: Optional[StrictStr]
    is_cloud_viewable: Optional[bool]
    auth: Optional[Dict[str, Any]]
    plan: Optional[Dict[str, Any]]
    usage: Optional[Dict[str, Any]]



from typing import Optional

from pydantic import Field, StrictInt

from polyaxon.schemas.base import BaseSchemaModel


class IntervalsConfig(BaseSchemaModel):
    runs_scheduler: Optional[StrictInt] = Field(alias="runsScheduler")
    operations_default_retry_delay: Optional[StrictInt] = Field(
        alias="operationsDefaultRetryDelay"
    )
    operations_max_retry_delay: Optional[StrictInt] = Field(
        alias="operationsMaxRetryDelay"
    )
    compatibility_check: Optional[StrictInt] = Field(alias="compatibilityCheck")



# import itertools
#
#
# class ForSchema(BaseCamelSchema):
#     class Meta:
#         ordered = True
#         fields = ("len", "do", "index")
#
#     @staticmethod
#     def schema_config():
#         return ForConfig
#
#
# class ForConfig(BaseConfig):
#     SCHEMA = ForSchema
#     IDENTIFIER = "for"
#
#     def __init__(self, len, do, index="index"):  # noqa, redefined-builtin `len`
#         self.len = len
#         self.do = do
#         self.index = index
#
#     def parse(self, parser, params):
#         parsed_data = []
#         length = parser.parse_expression(self.len, params, check_operators=True)
#         length = int(length)
#         for i in range(length):
#             i_params = {self.index: i}
#             i_params.update(params)
#             parsed_data.append(
#                 parser.parse_expression(self.do, i_params, check_operators=True)
#             )
#         if parsed_data and isinstance(parsed_data[0], (list, tuple)):
#             return list(itertools.chain.from_iterable(parsed_data))
#         return parsed_data
#
#
# class IfSchema(BaseCamelSchema):
#     class Meta:
#         ordered = True
#         fields = ("cond", "do", "else_do")
#
#     @staticmethod
#     def schema_config():
#         return IfConfig
#
#
# class IfConfig(BaseConfig):
#     SCHEMA = IfSchema
#     IDENTIFIER = "if"
#
#     def __init__(self, cond, do, else_do=None):
#         self.cond = cond
#         self.do = do
#         self.else_do = else_do
#
#     @staticmethod
#     def _check_cond(parser, cond):
#         cond_template = "{{% if {} %}}1{{% else %}}0{{% endif %}}"
#         cond_result = parser.parse_expression(cond_template.format(cond), {})
#         if int(cond_result) == 1:
#             return True
#         return False
#
#     def parse(self, parser, params):
#         cond = parser.parse_expression(self.cond, params, check_operators=False)
#         if self._check_cond(parser, cond):
#             return parser.parse_expression(self.do, params, check_operators=True)
#         if self.else_do:
#             return parser.parse_expression(self.else_do, params, check_operators=True)



from typing_extensions import Literal

from polyaxon.polyflow.run.job import V1Job
from polyaxon.polyflow.run.kinds import V1RunKind


class V1NotifierJob(V1Job):
    _IDENTIFIER = V1RunKind.NOTIFIER

    kind: Literal[_IDENTIFIER] = _IDENTIFIER



import sys

import click

from clipped.formatting import Printer
from clipped.utils.http import clean_host

from polyaxon import settings
from polyaxon.api import POLYAXON_CLOUD_HOST
from polyaxon.logger import clean_outputs


def get_dashboard_url(
    base: str = "ui", subpath: str = "", use_cloud: bool = False
) -> str:
    host = POLYAXON_CLOUD_HOST if use_cloud else clean_host(settings.CLIENT_CONFIG.host)
    dashboard_url = "{}/{}/".format(host, base)
    if subpath:
        return "{}{}/".format(dashboard_url, subpath.rstrip("/"))
    return dashboard_url


def get_dashboard(dashboard_url: str, url_only: bool, yes: bool):
    if url_only:
        Printer.header("The dashboard is available at: {}".format(dashboard_url))
        sys.exit(0)
    if yes or click.confirm(
        "Dashboard page will now open in your browser. Continue?",
        default=True,
    ):
        click.launch(dashboard_url)


@click.command()
@click.option(
    "--yes",
    "-y",
    is_flag=True,
    default=False,
    help="Automatic yes to prompts. "
    'Assume "yes" as answer to all prompts and run non-interactively.',
)
@click.option(
    "--url", is_flag=True, default=False, help="Print the url of the dashboard."
)
@clean_outputs
def dashboard(yes, url):
    """Open dashboard in browser."""
    get_dashboard(dashboard_url=get_dashboard_url(), url_only=url, yes=yes)
